@@ -4,16 +4,16 @@
  * Authors: Arun Raghavan <arun.raghavan@collabora.co.uk>
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
@@ -289,6 +289,9 @@ caps_from_video_stream_info (GstStreamVideoInformation *video)
         if (video->width)
                 gst_caps_set_simple (caps, "width", G_TYPE_INT,
                                 video->width, NULL);
+        if (video->depth)
+                gst_caps_set_simple (caps, "depth", G_TYPE_INT,
+                                video->depth, NULL);
         if (GST_VALUE_HOLDS_FRACTION (&video->frame_rate)) {
                 n = gst_value_get_fraction_numerator (&video->frame_rate);
                 d = gst_value_get_fraction_denominator (&video->frame_rate);
